@@ -15,11 +15,7 @@ class TestConnectionMethods(unittest.TestCase):
 
     @classmethod
     def tearDownClass(self):
-        self.tstcon.close()
-        with self.syscon.cursor() as c:
-            c.execute("DROP IDENT MIMERPY CASCADE")
-        self.syscon.commit()
-        self.syscon.close()
+        db_config.teardown(tstcon=self.tstcon, syscon=self.syscon)
 
     def tearDown(self):
         self.tstcon.commit()
@@ -368,7 +364,6 @@ class TestConnectionMethods(unittest.TestCase):
             try:
                 a = mimerpy.connect(**db_config.TSTUSR)
                 a.close()
-                #print("closing")
             except Exception as e:
                print("Error: in thread: ", e)
                """ """
@@ -398,7 +393,6 @@ class TestConnectionMethods(unittest.TestCase):
                 cur.execute("select * from threadbob")
                 cur.fetchone()
                 a.close()
-                #print("closing")
             except Exception as e:
                ###print("Error: in thread: ", e)
               """ """
@@ -454,7 +448,6 @@ class TestConnectionMethods(unittest.TestCase):
 
         a = mimerpy.connect(**db_config.TSTUSR)
         b = a.execute("select * from bob6")
-        print(b.fetchone())
         a.close()
 
 if __name__ == '__main__':
