@@ -17,11 +17,7 @@ class TestCursorMethods(unittest.TestCase):
 
     @classmethod
     def tearDownClass(self):
-        self.tstcon.close()
-        with self.syscon.cursor() as c:
-            c.execute("DROP IDENT MIMERPY CASCADE")
-        self.syscon.commit()
-        self.syscon.close()
+        db_config.teardown(tstcon=self.tstcon, syscon=self.syscon)
 
     def tearDown(self):
         self.tstcon.rollback()
@@ -108,7 +104,6 @@ class TestCursorMethods(unittest.TestCase):
             for val in range(10):
                 c.execute("insert into description values (?)", val)
             c.execute("select * from description")
-            #print(c.description)
             self.assertEqual(c.description,
                              (('columnone', 50, None, None, None, None, None),))
 
@@ -228,14 +223,17 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("create table bob13 (c1 INTEGER, c2 NVARCHAR(10),"
                       "                    c3 FLOAT) in pybank")
             self.tstcon.commit()
-            c.executemany("insert into bob13 values (:a, :b, :c)",
-                          ((1, 'pi', 14.345),
+            pmarkers = [(1, 'pi', 14.345),
                            (2, 'pii', 14.345),
                            (-3, 'piii', 14.345),
                            (7, 'piii', 14.345),
-                           (1121231, 'piiii', 14.345)))
+                           (1121231, 'piiii', 14.345)]
+            c.executemany("insert into bob13 values (:a, :b, :c)",
+                         pmarkers)
             self.tstcon.commit()
-            ## &&&& Fetch and test result?
+            c.execute("select * from bob13")
+            self.assertEqual(c.fetchall(),
+                            pmarkers)
 
     def test_insert_parametermarkers_russian(self):
         with self.tstcon.cursor() as c:
@@ -267,7 +265,7 @@ class TestCursorMethods(unittest.TestCase):
 
     def test_insert_parametermarkers_goodchar(self):
         with self.tstcon.cursor() as c:
-            c.execute("create table bobg14(c1 char(10)) in pybank");
+            c.execute("create table bobg14(c1 char(10)) in pybank")
             self.tstcon.commit()
             c.execute("insert into bobg14 values (?)", 'åååååååååå')
             self.tstcon.commit()
@@ -277,7 +275,7 @@ class TestCursorMethods(unittest.TestCase):
 
     def test_insert_parametermarkers_illchar(self):
         with self.tstcon.cursor() as c:
-            c.execute("create table bobz14(c1 char(10)) in pybank");
+            c.execute("create table bobz14(c1 char(10)) in pybank")
             self.tstcon.commit()
             with self.assertRaises(ProgrammingError):
                 c.execute("insert into bobz14 values (?)", '安排')
@@ -341,7 +339,6 @@ class TestCursorMethods(unittest.TestCase):
                           [(1, 'bob1'), (2, 'bob2'),
                            (3, 'bob3')])
 
-    ### &&&& Vad exakt testas nedan?
     def test_commit(self):
         with self.tstcon.cursor() as c:
             c.execute("create table bob7d (c1 INTEGER, c2 NVARCHAR(10))"
@@ -349,6 +346,7 @@ class TestCursorMethods(unittest.TestCase):
             c.executemany("insert into bob7d values (:a, :b)",
                           ((1, 'bob1'), (2, 'bob2'),
                            (3, 'bob3')))
+            self.tstcon.commit()
 
     ### &&&& Should fail since DDL has no commit
     def test_fetchone(self):
@@ -402,8 +400,7 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("select * from bob12")
             self.assertEqual(c.fetchmany(2),
                              [(9, 'bob9'), (10, 'bob10')])
-## &&&& Fixme: The following should work !?!
-##            self.assertEqual(c.fetchmany(2), [(11, 'bob11')])
+            self.assertEqual(c.fetchmany(2), [(11, 'bob11')])
 
     def test_fetchall(self):
         with self.tstcon.cursor() as c:
@@ -545,7 +542,7 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("insert into bob6567 values (3)")
             ## &&&& Should not fail
             with self.assertRaises(ProgrammingError):
-                c.execute("create table bob6566(c1 INTEGER) in pybank")
+                c.execute("create table bob65ss(c1 INTEGER) in pybank")
                 self.tstcon.rollback()
 
     def test_invalid_sequence_insert_parametermarkers(self):
@@ -558,7 +555,6 @@ class TestCursorMethods(unittest.TestCase):
                 c.execute("create table bob6566(c1 INTEGER) in pybank")
                 self.tstcon.rollback()
 
-    # Not 100% sure if InterfaceError is the correct error to be raised here
     def test_executemany_DDL(self):
         with self.tstcon.cursor() as c:
             with self.assertRaises(ProgrammingError):
@@ -652,7 +648,7 @@ class TestCursorMethods(unittest.TestCase):
             with self.assertRaises(DataError):
                 c.execute("insert INTO jon15 VALUES (?)", (nvar))
 
-    def test_invalid_int16_insert_too_small(self):
+    def test_invalid_int16_insert_too_big(self):
         with self.tstcon.cursor() as c:
             c.execute("create table jon17 (c1 SMALLINT) in pybank")
             nvar = 2 ** 15
@@ -670,9 +666,6 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("select * from jon16")
             self.assertEqual(c.fetchall(), [(0.6666666865348816, var)])
 
-
-    # &&&& Works now, I will have to look at how the test looked in older version - Erik 2018-08
-    # @unittest.skip
     def test_invalid_double_insert(self):
         b = self.tstcon.cursor()
         c = self.tstcon.cursor()
@@ -683,11 +676,13 @@ class TestCursorMethods(unittest.TestCase):
         var = 10 ** 309
         with self.assertRaises(DataError):
             b.execute("insert INTO jondd VALUES (?)", (var))
-
         with self.assertRaises(ProgrammingError):
             b.execute("insert INTO jondd VALUES (?, ?)", (-var, -var))
-
         b.close()
+        c.close()
+        d.close()
+        e.close()
+        f.close()
 
     def test_valid_double_insert_none(self):
         with self.tstcon.cursor() as c:
@@ -696,7 +691,8 @@ class TestCursorMethods(unittest.TestCase):
             var = None
             c.execute("insert INTO jon18 VALUES (?, ?)", (var, var))
             self.tstcon.commit()
-            ### &&&& Should fetch and check result
+            c.execute("select * from jon18")
+            self.assertEqual(c.fetchall(), [(None, None)])
 
     def test_valid_double_select_none(self):
         with self.tstcon.cursor() as c:
@@ -860,9 +856,6 @@ class TestCursorMethods(unittest.TestCase):
         c2 = b2.fetchall()
         self.assertEqual(len(c2), 2)
         self.assertEqual(c2, [(1,), (2,)])
-        # a1.commit()
-        # a2.commit()
-        # b3 = a2.cursor()
         b2.execute("SELECT * FROM jonisolated2")
         c3 = b2.fetchall()
         self.assertEqual(len(c3), 5)
@@ -883,7 +876,6 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("SELECT * FROM jonisolated3")
             c1 = c.fetchall()
             self.assertEqual(c1, [(1,), (2,), (3,), (4,), (5,), ])
-            # self.tstcon.commit()
             c.execute("SELECT * FROM jonisolated3")
             c2 = c.fetchall()
             self.assertEqual(c2, [(1,), (2,), (3,), (4,), (5,), ])
@@ -969,19 +961,15 @@ class TestCursorMethods(unittest.TestCase):
             r = c.fetchall()[0]
             self.assertEqual(r[0], b'0x53')
 
-    @unittest.skip
-    # &&&& need to find a picture to put in the repo
-    # Nä, gör en psudorandom-blob
     def test_insert_nclob_2(self):
         with self.tstcon.cursor() as c:
             c.execute("create table jonnclob2 (c1 Nclob(450000)) in pybank")
-            with open("../book2.txt", "r") as input_file:
-                anclob = input_file.read()
-                c.execute("insert INTO jonnclob2 VALUES (?)", (anclob))
-                self.tstcon.commit()
-                c.execute("select * from jonnclob2")
-                r = c.fetchall()[0]
-                self.assertEqual(r[0], anclob)
+            res = ''.join(format(i, 'b') for i in bytearray("Hello there", encoding ='utf-8')) 
+            c.execute("insert INTO jonnclob2 VALUES (?)", (res))
+            self.tstcon.commit()
+            c.execute("select * from jonnclob2")
+            r = c.fetchall()[0]
+            self.assertEqual(r[0], res)
 
     def test_insert_clob(self):
         with self.tstcon.cursor() as c:
@@ -1013,18 +1001,24 @@ class TestCursorMethods(unittest.TestCase):
             r = c.fetchall()[0]
             self.assertEqual(r[0], time)
     
-    @unittest.skip
-    # Currenntly not working...
     def test_insert_decimal(self):
         with self.tstcon.cursor() as c:
-            c.execute("create table jondecimal (c1 DECIMAL(5,2)) in pybank")
-            floatnum = '32423.234234'
+            c.execute("create table jondecimal (c1 DECIMAL(7,2)) in pybank")
+            floatnum = '32423.23'
             c.execute("insert INTO jondecimal VALUES (?)", (floatnum))
             self.tstcon.commit()
             c.execute("select * from jondecimal")
             r = c.fetchall()[0]
             self.assertEqual(r[0], floatnum)
 
+    @unittest.skipUnless(db_config.MIMERPY_STABLE == False, "Currently gives incorrect error message")
+    def test_insert_decimal_invalid(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table jondecimal2 (c1 DECIMAL(5,2)) in pybank")
+            floatnum = '32423.23234'
+            #with self.assertRaises(ProgrammingError):
+            c.execute("insert INTO jondecimal2 VALUES (?)", (floatnum))
+            
     def test_insert_bool(self):
         with self.tstcon.cursor() as c:
             c.execute("create table bobybool (c1 boolean) in pybank")
@@ -1164,6 +1158,7 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("select * from invalidbob")
             r = c.fetchall()
 
+
     def test_varchar_table(self):
         with self.tstcon.cursor() as c:
             c.execute("create table varcharbob(c1 VARCHAR(128)) in pybank")
@@ -1187,24 +1182,18 @@ class TestCursorMethods(unittest.TestCase):
             with self.assertRaises(NotSupportedError):
                 c.callproc()
 
-    # Blob's not working??? 2017-12-22
-    #  &&&& ?
-    @unittest.skip
-    def test_test_test(self):
+    def test_insert_blob_2(self):
         with self.tstcon.cursor() as c:
             c.execute("CREATE TABLE blob_table (blobcolumn BLOB)")
-
-            # reading from the .jpg file in binary mode
-            with open("../sphinx/mimerhd.png", 'rb') as input_file:
-                insert_blob = input_file.read()
-            c.execute("INSERT INTO blob_table VALUES (?)", (insert_blob))
-
-            # committing and closing the cursor and connection
+            blob = ''.join(format(i, 'b') for i in bytearray("Hello there again", encoding ='utf-8')) 
+            c.execute("INSERT INTO blob_table VALUES (?)", (blob))
             self.tstcon.commit()
             c.execute("SELECT * FROM blob_table")
-            self.assertEqual(c.fetchall()[0], (insert_blob,))
+            r = c.fetchall()[0][0].decode("utf-8")
+            self.assertEqual(r, blob)
 
-    def test_test_test2(self):
+    # these are the test examples for the documentaion
+    def test_for_doc_1(self):
         with self.tstcon.cursor() as c:
             c.execute("CREATE TABLE with_table_cursor1(c1 INTEGER,"
                       "                            c2 VARCHAR(32)) in pybank")
@@ -1214,13 +1203,12 @@ class TestCursorMethods(unittest.TestCase):
                       (2, "on how to use"))
             c.execute("INSERT INTO with_table_cursor1 VALUES (?,?)",
                       (3, "the with functionality."))
-
         with self.tstcon.cursor() as c:
             c.execute("SELECT * from with_table_cursor1")
 
     # these are the test examples for the documentaion
     @unittest.skip
-    def test_test_test3(self):
+    def test_for_doc_2(self):
         with self.tstcon.cursor() as c:
             c.execute("CREATE TABLE with_table_connection1(c1 INTEGER,"
                              " c2 VARCHAR(32)) in pybank")
@@ -1313,7 +1301,6 @@ create table longboi (c1 char(10),
                                             None, None, None, None, None,
                                             None, None, None, None, None, None))
 
-
     def test_data_type_varbinary(self):
         with self.tstcon.cursor() as c:
             long = "create table longboi_varbinary (c1 VARBINARY(10)) in pybank"
@@ -1361,6 +1348,39 @@ create table longboi (c1 char(10),
             c.execute("SELECT * from bob_name")
             self.assertEqual(c.fetchone(), (True, None, 3))
 
+    def test_error_message(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table bob_message (c1 nchar(10)) in pybank")
+            with self.assertRaises(ProgrammingError):
+                c.execute("insert into bob_message (c1,c3) values (:a,:b)",
+                        {'a':2,})
+            print()
+            self.assertEqual(c.messages[0][1], (-12202, 'c3 is not a column of an inserted table, updated table or any table identified in a FROM clause'))
+
+    def test_error_exception_errno(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table bob_message_1 (c1 nchar(10)) in pybank")
+            with self.assertRaises(ProgrammingError) as e:
+                c.execute("insert into bob_message_1 (c1,c3) values (:a,:b)",
+                        {'a':2,})
+            self.assertEqual(e.exception.errno, -12202)
+    
+    def test_error_exception_message(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table bob_message_2 (c1 nchar(10)) in pybank")
+            with self.assertRaises(ProgrammingError) as e:
+                c.execute("insert into bob_message_2 (c1,c3) values (:a,:b)",
+                        {'a':2,})
+            self.assertEqual(e.exception.message, "c3 is not a column of an inserted table, updated table or any table identified in a FROM clause")
+    
+    def test_error_exception_str(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table bob_message_3 (c1 nchar(10)) in pybank")
+            with self.assertRaises(ProgrammingError) as e:
+                c.execute("insert into bob_message_3 (c1,c3) values (:a,:b)",
+                        {'a':2,})
+            self.assertEqual(str(e.exception), "-12202 c3 is not a column of an inserted table, updated table or any table identified in a FROM clause")
+
     def test_parameter_name_2(self):
         with self.tstcon.cursor() as c:
             c.execute("create table bob_name_2(c1 boolean, c2 nchar(10),"
